import java.awt.*;
import java.util.Random;

// Класс, отвечающий за прорисовку и перемещение объекта-сущности
public class DrawningTracktor {
    private EntityTracktor entityTracktor;            // Класс-сущность
    public EntityTracktor getEntityTracktor(){
        return entityTracktor;
    }
    protected void setEntityTracktor(EntityTracktor entityTracktor){
        this.entityTracktor = entityTracktor;
    }
    public IMoveableObject GetMoveableObject(){
        return  new DrawningObjectExcavator(this);
    }
    protected int _startPosX;                   // Левая координата отрисовки трактора
    protected int _startPosY;                   // Верхняя кооридната отрисовки трактора
    private Integer _pictureWidth;       // Ширина окна отрисовки
    private Integer _pictureHeight;      // Высота окна отрисовки
    protected int _tracktorWidth = 110;     // Ширина отрисовки трактора
    protected int _tracktorHeight = 87;     // Высота отрисовки трактора
    public int getPosX(){
        return _startPosX;
    }
    public int getPosY(){
        return _startPosY;
    }
    public int getWidth(){
        return _tracktorWidth;
    }
    public int getHeight(){
        return _tracktorHeight;
    }
    private IDrawningRollers drawningRollers;

    // Инициализация свойств
    public DrawningTracktor(int speed, double weight, Color bodyColor,int width,int height, int countRollers)
    {
        if(height < _tracktorHeight || width < _tracktorWidth ){
            return;
        }
        _pictureWidth = width;
        _pictureHeight = height;
        entityTracktor = new EntityTracktor(speed, weight,bodyColor);
        Random random = new Random();
        drawningRollers = switch (random.nextInt(0, 3)) {
            case 0 -> new DrawningRollers(random.nextInt(0,3), bodyColor);
            case 1 -> new DrawningSquaredRollers(random.nextInt(0,3), bodyColor);
            case 2 -> new DrawningCrossRollers(random.nextInt(0,3), bodyColor);
            default -> new DrawningRollers(random.nextInt(0,3), bodyColor);
        };
        drawningRollers.setRollersCount((countRollers));

    }
    protected DrawningTracktor(int speed, double weight, Color bodyColor, int width, int height, int tracktorWidth,int tracktorHeight, int countRollers){
        if(width < tracktorWidth || height < tracktorHeight)
            return;
        _pictureWidth = width;
        _pictureHeight = height;
        _tracktorWidth = tracktorWidth;
        _pictureHeight = tracktorHeight;
        entityTracktor = new EntityTracktor(speed,weight,bodyColor);
        Random random = new Random();
        drawningRollers = switch (random.nextInt(0, 3)) {
            case 0 -> new DrawningRollers(random.nextInt(0,3), bodyColor);
            case 1 -> new DrawningSquaredRollers(random.nextInt(0,3), bodyColor);
            case 2 -> new DrawningCrossRollers(random.nextInt(0,3), bodyColor);
            default -> new DrawningRollers(random.nextInt(0,3), bodyColor);
        };
        drawningRollers.setRollersCount(countRollers);
    }
    public DrawningTracktor (EntityTracktor entityTracktor, int width, int height , int rollersCount, IDrawningRollers iDrawningRollers){
        if(height < _tracktorHeight || width < _tracktorWidth){
            return;
        }
        _pictureWidth = width;
        _pictureHeight = height;
        this.entityTracktor = entityTracktor;
        Random random = new Random();
        drawningRollers = iDrawningRollers;
        drawningRollers.setRollersCount(rollersCount);
    }
    public boolean CanMove(Direction direction){
        if(entityTracktor == null)
            return false;
        return switch (direction){
            case Left -> _startPosY - entityTracktor.getStep() > 0;
            case Up -> _startPosY - entityTracktor.getStep() > 0;
            case Right -> _startPosX + _tracktorHeight + entityTracktor.getStep() < _pictureWidth;
            case Down -> _startPosX + _tracktorWidth + entityTracktor.getStep() < _pictureHeight;
            default ->  false;
        };
    }

    // Установка позиции Трактора
    public void SetPosition(int x, int y)
    {
        if (x + _tracktorWidth > _pictureWidth){
            x = 0;
        }
        if( x < 0 || y + _tracktorHeight > _pictureHeight || y < 0)
        {
            y = 0;
        }
        _startPosX = x;
        _startPosY = y;
    }

    // Изменение направления перемещения
    public void MoveTransport(Direction direction)
    {
        if (!CanMove(direction) || entityTracktor == null)
        {
            return;
        }
        switch (direction)
        {
            // вправо
            case Right:
                if (_startPosX + _tracktorWidth + entityTracktor.getStep() < _pictureWidth)
                {
                    _startPosX += (int)entityTracktor.getStep();
                }
                break;
            //влево
            case Left:
                if (_startPosX - entityTracktor.getStep() > 0)
                {
                    _startPosX -= (int)entityTracktor.getStep();
                }
                break;
            //вверх
            case Up:
                if (_startPosY - entityTracktor.getStep() > 0)
                {
                    _startPosY -= (int)entityTracktor.getStep();
                }
                break;
            //вниз
            case Down:
                if (_startPosY + _tracktorHeight + entityTracktor.getStep() < _pictureHeight)
                {
                    _startPosY += (int)entityTracktor.getStep();
                }
                break;
        }
    }

    // Отрисовка Трактора
    public void DrawTransport(Graphics2D g)
    {
        if (_startPosX < 0 || _startPosY < 0 || _pictureHeight == null || _pictureWidth == null)
        {
            return;
        }
        Color penColor = Color.BLACK;

        // корпус
        Color br = entityTracktor!=null ? entityTracktor.getBodyColor() : Color.GRAY ;
        g.setColor(br);
        g.fillRect((int)_startPosX + 10, (int)_startPosY + 30, 90, 25);
        g.setColor(penColor);
        g.drawRect((int)_startPosX + 10, (int)_startPosY + 30, 90, 25);

        // окно
        g.setColor(Color.CYAN);
        g.fillRect((int)_startPosX + 65, (int)_startPosY + 1, 30, 29);
        g.setColor(penColor);
        g.drawRect((int)_startPosX + 65, (int)_startPosY + 1, 30, 29);

        // труба
        g.setColor(Color.RED);
        g.fillRect((int)_startPosX + 30, (int)_startPosY + 10, 10, 20);
        g.setColor(penColor);
        g.drawRect((int)_startPosX + 30, (int)_startPosY + 10, 10, 20);

        // гусеница
        g.setColor(Color.DARK_GRAY);
        g.fillOval((int)_startPosX + 1, (int)_startPosY + 57, 20, 20);
        g.setColor(penColor);
        g.drawOval((int)_startPosX + 1, (int)_startPosY + 57, 20, 20);
        g.setColor(Color.DARK_GRAY);
        g.fillOval((int)_startPosX + 1, (int)_startPosY + 65, 20, 20);
        g.setColor(penColor);
        g.drawOval((int)_startPosX + 1, (int)_startPosY + 65, 20, 20);
        g.setColor(Color.DARK_GRAY);
        g.fillOval((int)_startPosX + 90, (int)_startPosY + 57, 20, 20);
        g.setColor(penColor);
        g.drawOval((int)_startPosX + 90, (int)_startPosY + 57, 20, 20);
        g.setColor(Color.DARK_GRAY);
        g.fillOval((int)_startPosX + 90, (int)_startPosY + 65, 20, 20);
        g.setColor(penColor);
        g.drawOval((int)_startPosX + 90, (int)_startPosY + 65, 20, 20);
        g.setColor(Color.DARK_GRAY);
        g.fillRect((int)_startPosX + 10, (int)_startPosY + 57, 90, 30);
        g.fillRect((int)_startPosX + 1, (int)_startPosY + 65, 110, 10);
        g.setColor(penColor);
        g.drawLine((int)_startPosX + 10, (int)_startPosY + 57, (int)_startPosX + 100, (int)_startPosY + 57);
        g.drawLine((int)_startPosX + 10, (int)_startPosY + 86, (int)_startPosX + 100, (int)_startPosY + 86);
        g.drawLine((int)_startPosX + 1, (int)_startPosY + 65, (int)_startPosX + 1, (int)_startPosY + 75);
        g.drawLine((int)_startPosX + 110, (int)_startPosY + 65, (int)_startPosX + 110, (int)_startPosY + 75);

        drawningRollers.DrawRollers(g,_startPosX, _startPosY);
    }
}